@@ -3,10 +3,8 @@
 use strict;
 use warnings;
 package Alien::GCrypt;
-BEGIN {
-  $Alien::GCrypt::AUTHORITY = 'cpan:AJGB';
-}
-$Alien::GCrypt::VERSION = '1.6.2.1';
+our $AUTHORITY = 'cpan:AJGB';
+$Alien::GCrypt::VERSION = '1.6.5.0';
 use Alien::GPG::Error;
 
 use parent 'Alien::Base';
@@ -25,7 +23,7 @@ Alien::GCrypt - Install and make available libgcrypt
 
 =head1 VERSION
 
-version 1.6.2.1
+version 1.6.5.0
 
 =head1 SYNOPSIS
 
@@ -36,7 +34,7 @@ version 1.6.2.1
 
 =head1 DESCRIPTION
 
-Alien::GCrypt installs the C library C<libgcrypt>.
+Alien::GCrypt installs the C library C<libgcrypt> v1.6.5.
 
 =head1 SEE ALSO
 
@@ -52,7 +50,7 @@ Alex J. G. Burzyński <ajgb@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Alex J. G. Burzyński <ajgb@cpan.org>.
+This software is copyright (c) 2016 by Alex J. G. Burzyński <ajgb@cpan.org>.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
